/**
 * ${moduleNameUpperCase} Generated Driver API Header File
 *
 * @file ${moduleNameLowerCase}.h
 *
 * @defgroup ${moduleNameLowerCase}_host ${moduleNameUpperCase}_HOST
 *
 * @brief This header file provides APIs for the ${moduleNameUpperCase} driver.
 *
 * @version ${moduleNameUpperCase} Driver Version 2.1.0
*/

${disclaimer}

#ifndef ${moduleNameUpperCase}_H
#define ${moduleNameUpperCase}_H

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "${moduleGroupNameLowerCase}_host_types.h"
#include "${moduleGroupNameLowerCase}_host_interface.h"
#include "${moduleGroupNameLowerCase}_host_event_types.h"

#define ${componentNameLowerCase}_host_interface ${componentName}

<#if componentNameLowerCase!=moduleNameLowerCase>

#define ${componentName}_Initialize ${moduleNameUpperCase}_Initialize
#define ${componentName}_Deinitialize ${moduleNameUpperCase}_Deinitialize
#define ${componentName}_Write ${moduleNameUpperCase}_Write
#define ${componentName}_Read ${moduleNameUpperCase}_Read
#define ${componentName}_WriteRead ${moduleNameUpperCase}_WriteRead
#define ${componentName}_ErrorGet ${moduleNameUpperCase}_ErrorGet
#define ${componentName}_IsBusy ${moduleNameUpperCase}_IsBusy
#define ${componentName}_CallbackRegister ${moduleNameUpperCase}_CallbackRegister
#define ${componentName}_Tasks ${moduleNameUpperCase}_Tasks
</#if>

/**
 Section: Data Type Definitions
*/

extern const ${moduleGroupNameLowerCase}_host_interface_t ${componentName};

#define ${moduleNameUpperCase}_BAUD(F_SCL, T_RISE)    \
    ((((((float)${CLK_PER} / (float)F_SCL)) - 10 - ((float)${CLK_PER} * T_RISE / 1000000))) / 2)


/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API initializes the ${componentName} driver.
 * @param none
 * @return none
 */ 
void ${moduleNameUpperCase}_Initialize(void);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API Deinitializes the ${componentName} driver. This routine disables the ${componentName} module.
 * @param none
 * @return none
 */
void ${moduleNameUpperCase}_Deinitialize(void);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API writes data to a Client on the bus.
 * @param[in] uint16_t address  - 7-bit / 10-bit Client address.
 * @param[in] uint8_t *data     - pointer to source data buffer that contains the data to be transmitted.
 * @param[in] size_t dataLength - length of data buffer in number of bytes. Also the number of bytes to be written.
 * @retval true  - The request was placed successfully and the bus activity was initiated
 * @retval False - The request fails,if there was already a transfer in progress when this function was called
 */
bool ${moduleNameUpperCase}_Write(uint16_t address, uint8_t *data, size_t dataLength);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API reads the data from a client on the bus.
 * @param[in] uint16_t address  - 7-bit / 10-bit Client address.
 * @param[out] uint8_t *data    - pointer to destination data buffer that contains the data to be received
 * @param[in] size_t dataLength - length of data buffer in number of bytes. Also the number of bytes to be read.
 * @retval true  - The request was placed successfully and the bus activity was initiated
 * @retval False - The request fails,if there was already a transfer in progress when this function was called
 */
bool ${moduleNameUpperCase}_Read(uint16_t address, uint8_t *data, size_t dataLength);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API writes data from the writeData to the bus and then reads data from the Client 
 *        and stores the received in the readData.
 * @param[in]  uint16_t address    - 7-bit / 10-bit Client address.
 * @param[out] uint8_t *writeData  - pointer to write data buffer.
 * @param[in]  size_t writeLength  - write data length in bytes.
 * @param[in]  uint8_t *readData   - pointer to read data buffer.
 * @param[in]  size_t readLength   - read data length in bytes.
 * @retval true  - The request was placed successfully and the bus activity was initiated
 * @retval False - The request fails,if there was already a transfer in progress when this function was called
 */
bool ${moduleNameUpperCase}_WriteRead(uint16_t address, uint8_t *writeData, size_t writeLength, uint8_t *readData, size_t readLength);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This function get the error occurred during I2C Transmit and Receive.
 * @param none
 * @retval  I2C_ERROR_NONE - No Error
 * @retval  I2C_ERROR_NACK - Client returned NACK
 * @retval  I2C_ERROR_BUS_COLLISION - Bus Collision Error
 */
${moduleGroupNameLowerCase}_host_error_t ${moduleNameUpperCase}_ErrorGet(void);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This API checks if I2C is busy.
 * @param none
 * @retval true  - I2C is busy
 * @retval false - I2C is free
 */
bool ${moduleNameUpperCase}_IsBusy(void);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief This is polling function for non interrupt mode.
 * @param none
 * @return none
 */
void ${moduleNameUpperCase}_Tasks(void);

/**
 * @ingroup ${moduleGroupNameLowerCase}_host
 * @brief Setter function for I2C interrupt callback, This will be called when any error is generated.
 * @param void *CallbackHandler - Pointer to custom Callback.
 * @return none
 */
void ${moduleNameUpperCase}_CallbackRegister(void (*callbackHandler)(void));

#endif //${moduleNameUpperCase}_H
