import { InterfaceId } from "@microchip/scf-common/lib/InterfaceId";
import { ExportInterfaces } from "@microchip/scf-automodule-interface/lib/src/InterfaceTypes";
import { MyExportNames, MyExports } from "../generated_module/src/types";
export type ArgsList<ArgType = any> = {
    [key: string]: ArgType;
};
export type InterfaceInfo = {
    exportId: MyExportNames;
    interfaceId: InterfaceId;
};
export type InterfacesInfo = {
    [key: string]: InterfaceInfo;
};
export class ArgsProcessorHelper<Arguments> {
    private exportInterfaces: ExportInterfaces<MyExports, MyExportNames>;
    readonly interfacesInfo: InterfacesInfo;

    constructor(
        exportInterfaces: ExportInterfaces<MyExports, MyExportNames>,
        interfacesInfo: InterfacesInfo,
    ) {
        this.exportInterfaces = exportInterfaces;
        this.interfacesInfo = interfacesInfo;
    }

    public getArgsList = (...keys: string[]): ArgsList<Arguments> => {
        let interfacesArgs: ArgsList<Arguments> = {};

        keys.forEach((key) => {
            const interfaceInfo = this.interfacesInfo[key];
            if (interfaceInfo) {
                interfacesArgs = {
                    ...interfacesArgs,
                    ...this.exportInterfaces?.getInterface(
                        interfaceInfo?.exportId,
                        interfaceInfo?.interfaceId,
                    )?.args,
                };
            }
        });
        return interfacesArgs;
    };

    public getProcessedArg = (
        processorFnc: (args: Arguments) => string | boolean | number | undefined,
        ...interfaceIds: string[]
    ): string | boolean | number | undefined => {
        const argsList = this.getArgsList(...interfaceIds);
        let processedValue: string | boolean | number | undefined;
        if (argsList) {
            const argsKeys = Object.keys(argsList);

            argsKeys.some((key) => {
                const args: Arguments = argsList[key];
                if (args) processedValue = processorFnc(args);
                if (processedValue != undefined) return true;
            });
        }

        return processedValue;
    };
}
