export interface ClockParameters {
    clkPerInHz: number;
    tRiseInUs: number;
    fSclInHz: number;
    regValue?: number;
}

export class ClockCalculations {
    private readonly params: ClockParameters;
    private actualClockSpeed: number;

    constructor(params: ClockParameters) {
        this.params = params;
    }

    public getMBAUD = (): number => {
        if (this.params.regValue == undefined) {
            const clkDivFscl = this.params.clkPerInHz / this.params.fSclInHz;
            const clkMulTrise = this.params.clkPerInHz * (this.params.tRiseInUs / 1e6);
            const num = clkDivFscl - 10 - clkMulTrise;
            let result = Math.floor(num / 2);
            if (result > 255) result = 255;
            if (result < 0) result = 0;
            this.params.regValue = result;
        }
        return this.params.regValue;
    };

    public getActualClockSpeed = (): number => {
        if (this.actualClockSpeed == undefined) {
            const clkMulTrise = this.params.clkPerInHz * (this.params.tRiseInUs / 1e6);
            const den = clkMulTrise + 10 + this.getMBAUD() * 2;
            const result = Math.floor(this.params.clkPerInHz / den);
            this.actualClockSpeed = result;
        }
        return this.actualClockSpeed;
    };
}
