import { getModel } from "../generated_module/src/App/AppModel";
import { DerivedData } from "@microchip/scf-automodule-interface/lib/src/DerivedData";
import { mockState } from "../generated_module/tests/AppModel.test";
import { mockAssignedModule } from "../generated_module/tests/processor.test";
import { getDerivedData } from "./DerivedData";
import * as MyDerivedData from "./DerivedData";
import { AppModel } from "../generated_module/src/types/AutoModuleTypes";

describe("Export Interface Test Cases ", () => {
    describe("Host mode-->i2c-host-basic", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const payload = derivedFunctions["i2c-host-basic_payload"]?.();
        it("Test for twi interface api payload", () => {
            expect(payload).toBeDefined();
        });
    });

    describe("Client mode-->i2c-host-basic", () => {
        let mockModuleClient = mockAssignedModule(mockState());

        mockModuleClient = {
            ...mockModuleClient,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Client",
                    },
                },
            },
        };

        const model = getModel(mockModuleClient, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const payload = derivedFunctions["i2c-client-basic_payload"]?.();
        it("Test for twi interface api payload", () => {
            expect(payload).toBeDefined();
        });
    });
});

describe("Import Interface test Cases ", () => {
    describe("Host DerivedData initializer_system test cases -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();

        const initializer_system_results = {
            initializer: {
                name: "TWI0_Initialize",
                header: {
                    name: "twi0.h",
                    path: "i2c_host/",
                },
            },
        };
        it("Test for initializer_system interface results", () => {
            expect(derivedFunctions?.initializer_system_results?.()).toEqual(
                initializer_system_results,
            );
        });
    });
    describe("Host DerivedData interrupt-standard import interface test cases -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        it("Test for interrupt_standard import interface name and version defined", () => {
            expect(model?.getImports()?.interrupt_standard?.interfaceId.name).toEqual(
                "interrupt-standard",
            );
            expect(model?.getImports()?.interrupt_standard?.interfaceId.version).toEqual(
                "1.0.1",
            );
        });

        it("Test for interrupt_standard import interface arguments", () => {
            const args = undefined;
            expect(derivedFunctions["interrupt_standard_args"]?.()).toStrictEqual(args);
        });

        it("Test for interrupt_standard import interface results", () => {
            const res = {
                interruptList: [
                    {
                        name: "APIE",
                        module: "TWI0",
                        isEnabled: false,
                        isrCallback: {
                            name: "TWI0undefined",
                            header: {
                                name: "twi0.h",
                                path: "include/",
                            },
                        },
                        generateISR: false,
                        isHighPriority: true,
                    },
                    {
                        name: "DIE",
                        module: "TWI0",
                        isEnabled: false,
                        isrCallback: {
                            name: "TWI0undefined",
                            header: {
                                name: "twi0.h",
                                path: "include/",
                            },
                        },
                        generateISR: false,
                        isHighPriority: true,
                    },
                    {
                        name: "PIE",
                        module: "TWI0",
                        isEnabled: false,
                        isrCallback: {
                            name: "TWI0undefined",
                            header: {
                                name: "twi0.h",
                                path: "include/",
                            },
                        },
                        generateISR: false,
                        isHighPriority: true,
                    },
                    {
                        name: "RIE",
                        module: "TWI0",
                        isEnabled: false,
                        isrCallback: {
                            name: "TWI0undefined",
                            header: {
                                name: "twi0.h",
                                path: "include/",
                            },
                        },
                        generateISR: false,
                        isHighPriority: true,
                    },
                    {
                        name: "WIE",
                        module: "TWI0",
                        isEnabled: false,
                        isrCallback: {
                            name: "TWI0undefined",
                            header: {
                                name: "twi0.h",
                                path: "include/",
                            },
                        },
                        generateISR: false,
                        isHighPriority: true,
                    },
                ],
                interruptActions: {
                    APIE: {
                        isEnabled: [
                            {
                                validFor: true,
                                action: {
                                    type: "[FORM] SUBMIT",
                                    payload: {
                                        tabId: "main",
                                        value: true,
                                        path: "interrupt.apienSctrla",
                                    },
                                },
                            },
                        ],
                    },
                    DIE: {
                        isEnabled: [
                            {
                                validFor: true,
                                action: {
                                    type: "[FORM] SUBMIT",
                                    payload: {
                                        tabId: "main",
                                        value: true,
                                        path: "interrupt.dienSctrla",
                                    },
                                },
                            },
                        ],
                    },
                    PIE: {
                        isEnabled: [
                            {
                                validFor: true,
                                action: {
                                    type: "[FORM] SUBMIT",
                                    payload: {
                                        tabId: "main",
                                        value: true,
                                        path: "interrupt.pienSctrla",
                                    },
                                },
                            },
                        ],
                    },
                    RIE: {
                        isEnabled: [
                            {
                                validFor: true,
                                action: {
                                    type: "[FORM] SUBMIT",
                                    payload: {
                                        tabId: "main",
                                        value: true,
                                        path: "interrupt.rienMctrla",
                                    },
                                },
                            },
                        ],
                    },
                    WIE: {
                        isEnabled: [
                            {
                                validFor: true,
                                action: {
                                    type: "[FORM] SUBMIT",
                                    payload: {
                                        tabId: "main",
                                        value: true,
                                        path: "interrupt.wienMctrla",
                                    },
                                },
                            },
                        ],
                    },
                },
                moduleName: "TWI0",
            };
            expect(
                model?.getImportInterfaces().createProcessedImports("results")
                    ?.interrupt_standard?.results,
            ).toStrictEqual(res);
        });
    });

    describe("Host DerivedData pin_standard import interface test cases -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        it("Test for pin_standard import interface name and version defined", () => {
            expect(model?.getImports()?.pin_standard?.interfaceId.name).toEqual(
                "pin-standard",
            );
            expect(model?.getImports()?.pin_standard?.interfaceId.version).toEqual(
                "^0.1.0",
            );
        });

        it("Test for pin_standard import interface arguments", () => {
            const args = {
                queryAdapters: ["portmux"],
            };
            expect(derivedFunctions["pin_standard_args"]?.()).toStrictEqual(args);
        });
    });
});

describe("Calculations test Cases ", () => {
    describe("I2C Clock Calculation without fast mode-- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                        fmpenCtrla: true,
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        it("Test for DerivedData", () => {
            expect(derivedData).toBeDefined();
            expect(derivedData?.getModel()).toBeDefined();
        });
    });

    describe("I2C Clock Calculation with fast mode-- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                        fmpenCtrla: true,
                    },
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;

        it("Test for DerivedData", () => {
            expect(derivedData).toBeDefined();
            expect(derivedData?.getModel()).toBeDefined();
        });
    });
});

describe("Template Data test Cases ", () => {
    describe("DerivedData for TemplateData -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
            imports: {
                ...(mockModuleHost?.imports ?? {}),
                interrupt_standard: {
                    ...(mockModuleHost?.imports?.interrupt_standard ?? {}),
                    payload: {
                        isVectoredInterrupt: true,
                        vectoredInterruptEnable: false,
                        ivtBaseAddress: 8,
                        hasPriority: true,
                        isHighPriority: {
                            INT: true,
                            OVF: false,
                        },
                    },
                },
            },
        };
        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();

        it("Test for GetTemplateData value", () => {
            expect(derivedFunctions.templateData?.()).toBeDefined();
            expect(derivedFunctions?.pinData?.()).toStrictEqual({});
        });
    });

    describe("DerivedData for TemplateData when Registers and Interrupts are undefined  -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
            imports: {
                ...(mockModuleHost?.imports ?? {}),
                interrupt_standard: {
                    ...(mockModuleHost?.imports?.interrupt_standard ?? {}),
                    handle: undefined,
                    payload: undefined,
                    results: undefined,
                    args: undefined,
                },
                scf_avr8_twi_v2: {
                    ...(mockModuleHost?.imports?.scf_avr8_twi_v2 ?? {}),
                    handle: undefined,
                    payload: undefined,
                },
            },
        };
        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        it("Test for GetTemplateData value", () => {
            expect(derivedFunctions.templateData?.()).toBeDefined();
            expect(derivedFunctions["i2c-host-basic_payload"]?.()).toBeDefined();
            expect(
                derivedFunctions.overrideDefaultValues?.("componentName"),
            ).toStrictEqual("");
        });
    });

    describe("DerivedData for TemplateData when fosc is undefined  -- >", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                        interruptDriven: true,
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
            imports: {
                ...(mockModuleHost?.imports ?? {}),
                interrupt_standard: {
                    ...(mockModuleHost?.imports?.interrupt_standard ?? {}),
                    payload: {
                        isVectoredInterrupt: true,
                        hasPriority: true,
                    },
                },
                osc_clocks: {
                    ...(mockModuleHost?.imports?.osc_clocks ?? {}),

                    handle: {
                        ...(mockModuleHost?.imports?.osc_clocks?.handle ?? {}),

                        providerId: "module5",

                        exportId: "osc_clocks",

                        label: "module5 : osc_clocks",
                    },

                    payload: {
                        ...(mockModuleHost?.imports?.osc_clocks?.payload ?? {}),

                        fosc: "0",
                    },
                },
                scf_avr8_twi_v2: {
                    ...(mockModuleHost?.imports?.scf_avr8_twi_v2 ?? {}),
                    handle: undefined,
                    payload: {
                        ...(mockModuleHost?.imports?.scf_avr8_twi_v2?.payload ?? {}),
                        interrupts: undefined,
                        registers: undefined,
                        properties: undefined,
                    },
                },
            },
        };
        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        it("Test for GetTemplateData value", () => {
            expect(derivedFunctions.templateData?.()).toBeDefined();
            expect(derivedFunctions?.actualClockSpeed?.()).toBe(0);
            expect(derivedFunctions?.isHostModeInterrupt?.()).toBe(true);
            expect(derivedFunctions?.isClientModeInterrupt?.()).toBe(false);
        });
    });
});

describe("Empty Derived Data Test ", () => {
    const derivedData = MyDerivedData.getDerivedData((undefined as unknown) as AppModel);

    it("Get My Functions Test ", () => {
        expect(derivedData.getMyFunctions()).toStrictEqual({});
    });

    it("Get Model Test ", () => {
        expect(derivedData.getModel()).toStrictEqual(undefined);
    });
});

describe("Alert test Cases ", () => {
    it("Client mode alert", () => {
        // let mockModuleHost = mockAssignedModule(mockState());
        // const model = getModel(mockModuleHost, getDerivedData);
        // const derivedData = model?.getDerivedData() as DerivedData;
        // const derivedFunctions = derivedData?.getMyFunctions();
        // const alert = [{
        //     type: "hint",
        //     text: "Dual Mode driver is not supported in MCC.",
        // }];
        // const addrValiadate = { "minimum": 0, "maximum": 127, "multipleOf": 1 }
        // expect(derivedFunctions.alerts?.()).toMatchObject(alert);
        // expect(derivedFunctions.slaveAddressValidator?.()).toMatchObject(addrValiadate);
    });
    it("Host Fast mode alert", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                        interruptDriven: true,
                    },
                    hardware: {
                        mode: "Host",
                        fmpenCtrla: true,
                    },
                },
            },
        };
        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        if (model?.getImports()?.osc_clocks.payload) {
            model.getImports().osc_clocks = {
                ...model.getImports().osc_clocks,
                payload: {
                    fosc: "1000000",
                },
            };
        }
        const alert = [
            {
                text: "Dual Mode driver is not supported in MCC.",
                type: "hint",
            },
            {
                text:
                    "FMP mode is selected.The Peripheral Clock (CLK_PER) has to run at 16MHz or faster.",
                type: "warning",
            },
            {
                text: "Interrupt Driven drivers are selected, enable interrupts.",
                type: "warning",
            },
        ];
        const desClkSpeedValiadate = { minimum: 1, maximum: 1000000, multipleOf: 1 };
        expect(derivedFunctions.alerts?.()).toMatchObject(alert);
        expect(derivedFunctions.desiredClockSpeedValidator?.()).toMatchObject(
            desClkSpeedValiadate,
        );
    });
});

describe("Calculations test Cases ", () => {
    describe("Test for interruptComponentName()", () => {
        it("DerivedData for interruptComponentName() when interrupt disabled", () => {
            let mockModuleHost = mockAssignedModule(mockState());

            mockModuleHost = {
                ...mockModuleHost,
                state: {
                    main: {
                        software: {
                            componentName: "I2C1",
                            interruptDriven: true,
                        },
                        hardware: {
                            mode: "Host",
                        },
                        interrupt: {
                            rienMctrla: true,
                            wienMctrla: true,
                        },
                    },
                },
            };
            const model = getModel(mockModuleHost, getDerivedData);
            const derivedData = model?.getDerivedData() as DerivedData;
            const derivedFunctions = derivedData?.getMyFunctions();
            expect(derivedFunctions.interruptComponentName?.("")).toStrictEqual(
                undefined,
            );
        });

        it("DerivedData for intmode() when interrupt enabled", () => {
            let mockModuleHost = mockAssignedModule(mockState());

            mockModuleHost = {
                ...mockModuleHost,
                state: {
                    main: {
                        software: {
                            componentName: "I2C1",
                            interruptDriven: true,
                        },
                        hardware: {
                            mode: "Host",
                        },
                        interrupt: {
                            rienMctrla: true,
                        },
                    },
                },
            };
            const model = getModel(mockModuleHost, getDerivedData);
            const derivedData = model?.getDerivedData() as DerivedData;
            const derivedFunctions = derivedData?.getMyFunctions();

            expect(derivedFunctions.interruptComponentName?.("RIE")).toStrictEqual(
                "rienMctrla",
            );
        });
    });

    describe("Test Calculations when hardware is undefined", () => {
        let mockModuleHost = mockAssignedModule(mockState());
        mockModuleHost = {
            ...(mockModuleHost ?? {}),
            imports: {
                ...(mockModuleHost?.imports ?? {}),
                scf_avr8_twi_v2: {
                    ...(mockModuleHost?.imports?.scf_avr8_twi_v2 ?? {}),
                    payload: undefined,
                },
            },
        };
        const model = getModel(mockModuleHost, MyDerivedData.getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const templateData1 = derivedFunctions["i2c-client-basic_payload"]?.();

        it("Test for Module Name", () => {
            expect(templateData1).toBeDefined();
        });

        it("Test for Module Name", () => {
            expect(derivedFunctions.moduleName?.()).toStrictEqual("");
        });

        it("Test for Override Custom Name Default Value", () => {
            expect(
                derivedFunctions.overrideDefaultValues?.("componentName"),
            ).toStrictEqual("");
        });
    });

    describe("Desired Clock speed validations", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    software: {
                        componentName: "I2C1",
                    },
                    hardware: {
                        mode: "Host",
                    },
                },
            },
        };
        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        if (model?.getImports()?.osc_clocks.payload) {
            model.getImports().osc_clocks = {
                ...model.getImports().osc_clocks,

                payload: undefined,
            };
        }
        const desClkSpeedValiadate = { minimum: 1, maximum: NaN, multipleOf: 1 };
        expect(derivedFunctions.desiredClockSpeedValidator?.()).toMatchObject(
            desClkSpeedValiadate,
        );
    });

    describe("DerivedData undefined Hardware getActiveName  test cases -- >", () => {
        const mockModule = mockAssignedModule(mockState());
        const model = getModel(mockModule, getDerivedData);
        if (model) {
            model.getHardware = (): any => {
                return undefined;
            };
        }
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const output = derivedFunctions?.getActiveName?.();

        it("Test for overrideDefaultValues result", () => {
            expect(output).toBe("undefined_Peripheral");
        });
    });

    describe("DerivedData overrideDefaultValues undefined Hardware test cases -- >", () => {
        const mockModule = mockAssignedModule(mockState());
        const model = getModel(mockModule, getDerivedData);
        if (model) {
            model.getHardware = (): any => {
                return undefined;
            };
        }
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const output = derivedFunctions?.overrideDefaultValues?.("componentName");

        it("Test for overrideDefaultValues result", () => {
            expect(output).toBe("");
        });
    });

    describe("DerivedData overrideDefaultValues test cases -- >", () => {
        const mockModule = mockAssignedModule(mockState());
        const model = getModel(mockModule, getDerivedData);
        if (model) {
            model.getRegisterValues = (): any => {
                return undefined;
            };
        }
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const output = derivedFunctions?.overrideDefaultValues?.("componentName");

        it("Test for overrideDefaultValues result", () => {
            expect(output).toBe("I2C0");
        });
    });

    describe("DerivedData saddr  test cases -- >", () => {
        const mockModule = mockAssignedModule(mockState());
        const model = getModel(mockModule, getDerivedData);
        if (model) {
            model.getHardware = (): any => {
                return undefined;
            };
        }
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const output = derivedFunctions?.saddr?.();

        it("Test for saddr result", () => {
            expect(output).toBe("35");
        });
    });

    describe("DerivedData getExportInterfaces undefined  test cases -- >", () => {
        const mockModule = mockAssignedModule(mockState());
        const model = getModel(mockModule, getDerivedData);

        if (model) {
            model.getExportInterfaces = (): any => {
                return {
                    getInterfaceData: (): any => {
                        return [
                            {
                                interface: "i2c-host-basic",
                                name: "actualClockSpeed",
                                value: "100000",
                                module: "requestId",
                            },
                            {
                                interface: "i2c-host-basic",
                                name: "interruptDriven",
                                value: true,
                                module: "requestId",
                            },
                        ];
                    },
                };
            };
        }
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();
        const output1 = derivedFunctions?.saddr?.();

        it("Test for getClientAddress result", () => {
            expect(output1).toBe(undefined);
        });
    });
});

describe("Host Polling With Interrupt Driven True -- >", () => {
    let mockModuleHost = mockAssignedModule(mockState());

    mockModuleHost = {
        ...mockModuleHost,
        state: {
            main: {
                software: {
                    interruptDriven: true,
                },
                hardware: {
                    mode: "Host",
                },
            },
        },
    };
    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isHostModePolling?.();

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("Host Polling Mode change to Xyz -- >", () => {
    let mockModuleHost = mockAssignedModule(mockState());

    mockModuleHost = {
        ...mockModuleHost,
        state: {
            main: {
                software: {
                    interruptDriven: false,
                },
                hardware: {
                    mode: "Client",
                },
            },
        },
    };

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model)
        model.getComponentValue = (cmp: string): any => {
            return "Client";
        };

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isHostModePolling?.();

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("Client Polling Mode True case-- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.getComponentValue = (cmp: string): any => {
            return "Client";
        };
        model.isComponentValue = (cmp: string): any => {
            return true;
        };
    }
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isClientModePolling?.();

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("Client Polling Mode False case-- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.getComponentValue = (cmp: string): any => {
            return "Host";
        };
        model.isComponentValue = (cmp: string): any => {
            return false;
        };
    }
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isClientModePolling?.();

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("Client Interface Loaded-- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.getComponentValue = (cmp: string): any => {
            return "Host";
        };
        model.isComponentValue = (cmp: string): any => {
            return false;
        };
    }
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.isClientModePolling?.();

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("RienMctrla Client Interface Loaded-- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.isComponentValue = (cmp: string): any => {
            return false;
        };
    }
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.rienMctrla?.();
    const output1 = derivedFunctions?.wienMctrla?.();
    const output2 = derivedFunctions?.apienSctrla?.();
    const output3 = derivedFunctions?.dienSctrla?.();
    const output4 = derivedFunctions?.pienSctrla?.();

    it("Rien Host Polling", () => {
        expect(output).toBe(false);
        expect(output1).toBe(false);
        expect(output2).toBe(false);
        expect(output3).toBe(false);
        expect(output4).toBe(false);
    });
});

describe("FIlter Interrupts -- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);
    if (model) {
        model.isComponentValue = (cmp: string): any => {
            return "xyz";
        };
    }
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.filterInterrupts?.("xyz");

    it("ISR Host Polling", () => {
        expect(output).toBe(false);
    });
});

describe("Test for get Mode Functions -- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    let model = getModel(mockModule, getDerivedData);

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    model = undefined;
    const output = derivedFunctions?.getModeFunctions?.();

    it("Test Mode Functions", () => {
        expect(output).toBeUndefined();
    });
});

describe("Export Interface Test Cases ", () => {
    describe("Host mode-->i2c-host-basic", () => {
        let mockModuleHost = mockAssignedModule(mockState());

        mockModuleHost = {
            ...mockModuleHost,
            state: {
                main: {
                    hardware: {
                        mode: "Host",
                    },
                },
            },
            exports: {
                i2c_basic: {
                    interfaces: [
                        {
                            ...mockModuleHost?.exports?.i2c_basic?.interfaces?.[0],
                        },
                        {
                            ...mockModuleHost?.exports?.i2c_basic?.interfaces?.[1],
                            args: undefined,
                            interfaceId: {
                                name: "i2c-client-basic",
                                version: "2.0.0",
                            },
                        },
                    ],
                },
            },
        };

        const model = getModel(mockModuleHost, getDerivedData);
        const derivedData = model?.getDerivedData() as DerivedData;
        const derivedFunctions = derivedData?.getMyFunctions();

        //expect(derivedFunctions?.customName?.()).toBe("I2C1");
        expect(derivedFunctions?.mode?.()).toEqual("Host");
        expect(derivedFunctions?.moduleName?.()).toEqual("TWI0");
        expect(derivedFunctions?.getClientAddress?.()).toBeUndefined();
    });
});

describe("Client mode-->i2c-client-basic", () => {
    let mockModuleClient = mockAssignedModule(mockState());

    mockModuleClient = {
        ...mockModuleClient,
        state: {
            main: {
                hardware: {
                    mode: "Client",
                    clientAddress: 8,
                },
            },
        },
        exports: {
            i2c_basic: {
                interfaces: [
                    {
                        ...mockModuleClient?.exports?.i2c_basic?.interfaces?.[0],
                        args: undefined,
                    },
                    {
                        ...mockModuleClient?.exports?.i2c_basic?.interfaces?.[1],
                        interfaceId: {
                            name: "i2c-client-basic",
                            version: "2.0.0",
                        },
                    },
                ],
            },
        },
    };

    const model = getModel(mockModuleClient, getDerivedData);
    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();

    expect(derivedFunctions?.mode?.()).toEqual("Client");
    expect(derivedFunctions?.moduleName?.()).toEqual("TWI0");
    expect(derivedFunctions?.getClientAddress?.()).toBeUndefined();
});

describe("Test actualClockSpeed -- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);

    if (model) {
        model.getImportValue = (cmp: string): any => {
            return undefined;
        };
        model.getComponentValue = (cmp: string): any => {
            return undefined;
        };
    }

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.actualClockSpeed?.();

    it("Test Mode Functions", () => {
        expect(output).toBeDefined();
    });
});

describe("Test getFirstExportArg  -- >", () => {
    const mockModuleHost = mockAssignedModule(mockState());

    const mockModule = mockAssignedModule(mockModuleHost);
    const model = getModel(mockModule, getDerivedData);

    if (model) {
        model.getComponentValue = (cmp: string): any => {
            return undefined;
        };
    }

    const derivedData = model?.getDerivedData() as DerivedData;
    const derivedFunctions = derivedData?.getMyFunctions();
    const output = derivedFunctions?.["i2c-host-basic_payload"]?.();

    it("Test Mode Functions", () => {
        expect(output).toBeDefined();
    });
});
