import { getModel } from "../generated_module/src/App/AppModel";
import { mockState } from "../generated_module/tests/AppModel.test";
import { mockAssignedModule } from "../generated_module/tests/processor.test";
import { getDerivedData } from "./DerivedData";
import { getCustomName, getGeneratorModel } from "./GeneratorModel";

describe("Host mode-->i2c-host-basic when no arguments requested", () => {
    let mockModuleHost = mockAssignedModule(mockState());

    mockModuleHost = {
        ...mockModuleHost,
        state: {
            main: {
                software: {
                    componentName: "I2C1",
                },
            },
        },
        exports: {
            i2c_basic: {
                interfaces: [
                    {
                        interfaceId: {
                            name: "i2c-host-basic",
                            version: "1.0.0",
                        },
                        results: { customName: "I2C1_Host" },
                    },
                    {
                        interfaceId: {
                            name: "i2c-client-basic",
                            version: "1.0.0",
                        },
                    },
                ],
            },
        },
    };

    const model = getModel(mockModuleHost, getDerivedData);
    if (model) {
        getCustomName(model, { name: "i2c-host-basic", version: "1.0.0" });
    }
});

describe("Client mode-->i2c-host-basic when no arguments requested", () => {
    let mockModuleHost = mockAssignedModule(mockState());
    mockModuleHost = {
        ...mockModuleHost,
        state: {
            main: {
                software: {
                    componentName: "I2C1",
                },
                hardware: {
                    mode: "Client",
                },
            },
        },
        exports: {
            i2c_basic: {
                interfaces: [
                    {
                        interfaceId: {
                            name: "i2c-host-basic",
                            version: "1.0.0",
                        },
                        results: { customName: "I2C1_Host" },
                    },
                    {
                        interfaceId: {
                            name: "i2c-client-basic",
                            version: "1.0.0",
                        },
                        results: { customName: "I2C1_Client" },
                    },
                ],
            },
        },
    };

    const model = getModel(mockModuleHost, getDerivedData);

    if (model) {
        model.getComponentValue = (cmp: string): any => {
            return "Client";
        };
        getGeneratorModel(model);
    }
});

describe("Host mode-->i2c-host-basic when no arguments requested 1", () => {
    let mockModuleHost = mockAssignedModule(mockState());

    mockModuleHost = {
        ...mockModuleHost,
        state: {
            main: {
                software: {
                    componentName: "I2C1",
                },
            },
        },
        exports: {
            i2c_basic: {
                interfaces: [
                    {
                        interfaceId: {
                            name: "i2c-host-basic",
                            version: "1.0.0",
                        },
                        results: { customName: "@xyz" },
                    },
                    {
                        interfaceId: {
                            name: "i2c-client-basic",
                            version: "1.0.0",
                        },
                    },
                ],
            },
        },
    };

    const model = getModel(mockModuleHost, getDerivedData);
    if (model) {
        getCustomName(model, { name: "i2c-host-basic", version: "1.0.0" });
    }
});
