import * as Processor from "@microchip/scf-common/lib/Processor";
import { AppModel } from "../generated_module/src/types/AutoModuleTypes";

export interface I2CModeFunctions {
    getAlerts: () => Processor.Alert[];
}

export const getI2CModeFunctions = (dataModel: AppModel): I2CModeFunctions => {
    switch (dataModel.getComponentValue("mode")) {
        case "Host":
            return new I2CHost(dataModel);
        case "Client":
            return new I2CClient(dataModel);
        default:
            throw new Error("Invalid mode.");
    }
};

class I2CHost implements I2CModeFunctions {
    constructor(private dataModel: AppModel) {}

    public getAlerts = (): Processor.Alert[] => {
        const alerts: Processor.Alert[] = [];

        if (
            this.dataModel.isComponentValue("interruptDriven") === true &&
            this.dataModel.isComponentValue("rienMctrla") === false &&
            this.dataModel.isComponentValue("wienMctrla") === false
        ) {
            alerts.push({
                text: "Interrupt Driven drivers are selected, enable interrupts.",
                type: Processor.AlertTypes.Warning,
            });
        }

        return alerts;
    };
}

class I2CClient implements I2CModeFunctions {
    constructor(private dataModel: AppModel) {}

    public getAlerts = (): Processor.Alert[] => {
        const alerts: Processor.Alert[] = [];

        if (this.dataModel.isComponentValue("interruptDriven")) {
            if (
                this.dataModel.isComponentValue("dienSctrla") === false &&
                this.dataModel.isComponentValue("apienSctrla") === false
            ) {
                alerts.push({
                    text: "Interrupt Driven drivers are selected, enable interrupts.",
                    type: Processor.AlertTypes.Warning,
                });
            }
        }

        return alerts;
    };
}
